
import gconf

import gtk
try :
    import hildon
except :
    from hildongtk import hildon

class Configuration :

    def __init__ ( self , type ) :
        self._type = type
        self.read()

    def read ( self ) :
        client = gconf.client_get_default()

        self.homedir = client.get_string( "/apps/wifihood/basedir" ) or "/home/user/MyDocs"
        self.dbname = client.get_string( "/apps/wifihood/dbname" ) or "wifiscanner.db"
        self.mapsdir = client.get_string( "/apps/wifihood/maps" ) or "/home/user/MyDocs/.maps"
        self.mapclass = client.get_string( "/apps/wifihood/maptype" ) or "OpenStreetMap I"

        self.scan_period = client.get_float( "/apps/wifihood/scan-period" ) or 5

        self.store_log = client.get_bool( "/apps/wifihood/store-logfile" )

        self.use_mapper = client.get_bool( "/apps/wifihood/use-mapper" )
        self.store_gps = client.get_bool( "/apps/wifihood/store-gps" )
        if self.use_mapper :
            self.lat = client.get_float( "/apps/maemo/maemo-mapper/center_latitude" )
            self.lon = client.get_float( "/apps/maemo/maemo-mapper/center_longitude" )
            self.zoom = client.get_int( "/apps/maemo/maemo-mapper/zoom" )
        else :
            self.lat = client.get_float( "/apps/wifihood/latitude" )
            self.lon = client.get_float( "/apps/wifihood/longitude" )
            self.zoom = client.get_int( "/apps/wifihood/map-zoom" )

        if self._type == 'scanner' :
            self.zoom = client.get_int( "/apps/wifihood/%s-zoom" % self._type )

        if self.lat == 0.0 and self.lon == 0.0 :
            self.lat , self.lon = 40.416 , -3.683
        if self.zoom == 0 :
            self.zoom = 15

    def set_latlon ( self , ( lat , lon ) ) :
        if self.store_gps and not self.use_mapper :
            client = gconf.client_get_default()
            client.set_float( "/apps/wifihood/latitude" , lat )
            client.set_float( "/apps/wifihood/longitude" , lon )
        self.lat , self.lon = lat , lon

    def set_zoom ( self , zoom ) :
        if self._type == 'scanner' or ( self.store_gps and not self.use_mapper ) :
            client = gconf.client_get_default()
            client.set_int( "/apps/wifihood/%s-zoom" % self._type , zoom )
        self.zoom = zoom

    def save ( self , widget=None , event=None ) :
        """Saves all the editable configuration elements"""

        client = gconf.client_get_default()
        client.set_string( "/apps/wifihood/basedir" , self.homedir )
        client.set_string( "/apps/wifihood/dbname" , self.dbname )

        client.set_float( "/apps/wifihood/scan-period" , self.scan_period )
        client.set_bool( "/apps/wifihood/store-logfile" , self.store_log )
        client.set_bool( "/apps/wifihood/use-mapper" , self.use_mapper )
        client.set_bool( "/apps/wifihood/store-gps" , self.store_gps )

        client.set_int( "/apps/wifihood/%s-zoom" % self._type , self.zoom )

class SettingsWindow ( hildon.StackableWindow ) :

    def __init__ ( self , config , handler=None ) :

        hildon.StackableWindow.__init__( self )
        self.set_title( "Wifihood Settings" )

        self.connect_object("delete_event", config.save , self )

        scrollwin = hildon.PannableArea()
        scrollwin.show()
        self.add(scrollwin)

        vbox = gtk.VBox(False, 0)
        vbox.show()
        scrollwin.add_with_viewport( vbox )

        self.add_dataframe( vbox , config )
        self.add_mapsframe( vbox , config , handler )
        self.add_dbframe( vbox , config )

        self.show()


    def ZoomButton ( self , text , config , handler ) :
        selector = ZoomDialog( config , handler )
        button = hildon.PickerButton( gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_VERTICAL)
        button.set_title( text )
        button.set_selector( selector )
        # FIXME bug#6892 missing signal
        button.connect_object( "value-changed", selector.zoomdialog , button , config )
        return button

    def add_dbframe ( self , vbox , config ) :

        dbframe = gtk.Frame( label="Database" )
        dbframe.set_label_align(0 , 0.1)
        dbframe.show()
        vbox.pack_start(dbframe, True, True, 0)

        database = gtk.Table(4, 1, False)
        database.show()
        dbframe.add(database)

        dblabel = gtk.Label( "Home directory" )
        dblabel.show()
        database.attach(dblabel, 0, 1, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 5)

        dbvalue = hildon.Entry( gtk.HILDON_SIZE_FINGER_HEIGHT )
        dbvalue.set_width_chars(15)
        dbvalue.connect( "changed" , self.entry_cb , config , "homedir" )
        dbvalue.set_text( config.homedir )
        dbvalue.show()
        database.attach(dbvalue, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 5)

        dblabel = gtk.Label( "Database name" )
        dblabel.show()
        database.attach(dblabel, 2, 3, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 5)

        dbvalue = hildon.Entry( gtk.HILDON_SIZE_FINGER_HEIGHT )
        dbvalue.set_width_chars(12)
        dbvalue.connect( "changed" , self.entry_cb , config , "dbname" )
        dbvalue.set_text( config.dbname )
        dbvalue.show()
        database.attach(dbvalue, 3, 4, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 5)


    def add_mapsframe ( self , vbox , config , handler ) :

        mapsframe = gtk.Frame( label="Maps & Coordinates" )
        mapsframe.set_label_align(0 , 0.1)
        mapsframe.show()
        vbox.pack_start(mapsframe, True, True, 0)

        maps = gtk.Table(2, 2, False)
        maps.show()
        mapsframe.add(maps)

        button = hildon.CheckButton( gtk.HILDON_SIZE_FINGER_HEIGHT )
        button.set_label( "Use OpenStreet maps" )
        button.set_active( 1 )
        button.set_sensitive(0)
        button.show()
        maps.attach(button, 0, 1, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 5)

        zoomlevel = self.ZoomButton( "Zoom level" , config , handler )
        zoomlevel.show()
        maps.attach(zoomlevel, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 5)

        button = hildon.CheckButton( gtk.HILDON_SIZE_FINGER_HEIGHT )
        button.set_label( "Use maemo-mapper coordinates" )
        button.connect( "toggled" , self.checkbutton_cb , config , "use_mapper" )
        button.set_active( config.use_mapper )
        button.show()
        maps.attach(button, 0, 1, 1, 2, gtk.EXPAND|gtk.FILL, 0, 0, 5)

        button = hildon.CheckButton( gtk.HILDON_SIZE_FINGER_HEIGHT )
        button.set_label( "Store coordinates changes" )
        button.connect( "toggled" , self.checkbutton_cb , config , "store_gps" )
        button.set_active( config.store_gps )
        button.show()
        maps.attach(button, 1, 2, 1, 2, gtk.EXPAND|gtk.FILL, 0, 0, 5)


    def add_dataframe ( self , vbox , config ) :

        dataframe = gtk.Frame( label="Data gathering" )
        dataframe.set_label_align(0 , 0.1)
        dataframe.show()
        vbox.pack_start(dataframe, True, True, 0)

        datatable = gtk.Table(3, 1, False)
        datatable.show()
        dataframe.add(datatable)

        scanlabel = gtk.Label( "Scanning interval" )
        scanlabel.show()
        datatable.attach(scanlabel, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL)

        scanvalue = hildon.Entry( gtk.HILDON_SIZE_FINGER_HEIGHT )
        scanvalue.set_width_chars(5)
        scanvalue.connect( "changed" , self.float_cb , config , "scan_period" )
        scanvalue.set_text( "%s" % config.scan_period )
        scanvalue.show()
        datatable.attach(scanvalue, 2, 3, 0, 1, gtk.FILL)

        button = hildon.CheckButton( gtk.HILDON_SIZE_FINGER_HEIGHT )
        button.set_label( "Write full logfile" )
        button.connect( "toggled" , self.checkbutton_cb , config , "store_log" )
        button.set_active( config.store_log )
        button.show()
        datatable.attach(button, 0, 1, 0, 1, gtk.EXPAND|gtk.FILL)

    def entry_cb ( self , entry , config , keyword ) :
        config.__dict__[ keyword ] = entry.get_text()

    def float_cb ( self , entry , config , keyword ) :
        config.__dict__[ keyword ] = float( entry.get_text() )

    def int_cb ( self , entry , config , keyword , scale=1 ) :
        config.__dict__[ keyword ] = int( scale * float( entry.get_text() ) )

    def checkbutton_cb ( self , button , config , keyword ) :
        config.__dict__[ keyword ] = button.get_active()

class ZoomDialog ( hildon.TouchSelector ) :

    def __init__ ( self , config , handler ) :
        hildon.TouchSelector.__init__( self )
        self.handler = handler

        zooms = gtk.ListStore(str)

        active = index = 0
        for zoom in range(8,19) :
            iter = zooms.append()
            zooms.set( iter , 0 , "%2d" % zoom )
            if zoom == config.zoom :
                active = index
            index += 1

        column = self.append_text_column( zooms , True )

        # NOTE : with text=True, we must use 1 instead of 0
        self.set_active( 0 , active )

    def zoomdialog ( self , widget , conf ) :
        newzoom = int( widget.get_selector().get_current_text() )
        if self.handler : self.handler( newzoom )
        conf.set_zoom( newzoom )


if __name__ == "__main__" :
    config = Configuration( 'scanner' )
    window = SettingsWindow( config )
    window.connect("delete_event", gtk.main_quit, None)
    window.show()
    gtk.main()
