
try :
    import location
except :
    import hildongtk.location as location


import gobject

class GPSObject ( gobject.GObject ) :

    def __init__ ( self ) :
        gobject.GObject.__init__( self )

        self.control = location.GPSDControl.get_default()
        self.control.set_properties(preferred_method=location.METHOD_GNSS,                                                  
                                    preferred_interval=location.INTERVAL_DEFAULT)                                              
        self.device = location.GPSDevice()
        self.device.connect_object("changed", GPSObject.update , self)

        # Values to be set by GPSDevice changed events
        self.info = None, 0, 0, None, None, None, None
        self.satellites = None
        self.cells = None
        self.ngps = 0

    def set_method ( self , method="gps" ) :
        if method == "agps" :
            self.control.set_properties(preferred_method=location.METHOD_GNSS|location.METHOD_AGNSS)
        else :
            control.set_properties(preferred_method=location.METHOD_GNSS)

    def set_interval ( self , interval=0 ) :
        if interval <= 0 :
            self.control.set_properties(preferred_interval=location.INTERVAL_DEFAULT)
        elif interval <= 1 :
            self.control.set_properties(preferred_interval=location.INTERVAL_1S)
        elif interval <= 2 :
            self.control.set_properties(preferred_interval=location.INTERVAL_2S)
        elif interval <= 5 :
            self.control.set_properties(preferred_interval=location.INTERVAL_5S)
        elif interval <= 10 :
            self.control.set_properties(preferred_interval=location.INTERVAL_10S)
        elif interval <= 20 :
            self.control.set_properties(preferred_interval=location.INTERVAL_20S)
        elif interval <= 30 :
            self.control.set_properties(preferred_interval=location.INTERVAL_30S)
        elif interval <= 60 :
            self.control.set_properties(preferred_interval=location.INTERVAL_60S)
        else :
            self.control.set_properties(preferred_interval=location.INTERVAL_120S)

    def start ( self ) :
        if not self.device.online :
            self.control.start()

    # FIXME : Stopping does not work, at least while getting fix
    def stop ( self ) :
        if self.device.online :
            self.control.stop()

    def update ( self ) :

        if self.device.online :
            if self.device.status == location.GPS_DEVICE_STATUS_FIX :
                state = "FIX"
                self.ngps += 1
            elif self.device.status == location.GPS_DEVICE_STATUS_DGPS_FIX :
                state = "DGPS"
                # FIXME : Increase also ngps here ???
            else :
                state = None

            lat , lon , alt = None , None , None
            if self.device.fix[1] & location.GPS_DEVICE_LATLONG_SET:
                lat , lon = self.device.fix[4:6]
            if self.device.fix[1] & location.GPS_DEVICE_ALTITUDE_SET:
                alt = self.device.fix[7]
            # FIXME : get time from GPS fix
            self.info = state , self.device.satellites_in_view , self.device.satellites_in_use , self.device.fix[2] , lat , lon , alt
            self.satellites = self.device.satellites
            self.cells = self.device.cell_info
        else :
            self.info = None, 0, 0, None, None, None, None

    def __str__ ( self ) :
        output = map( str , self.info )
        return " ".join( output )

    def report ( self ) :
        return "%d gps" % self.ngps

gobject.type_register(GPSObject)

if __name__ == "__main__" :
    loop = gobject.MainLoop()
    sample = GPSObject()
    def on_stop(control, mainloop):
        mainloop.quit()
    sample.control.connect("gpsd-stopped", on_stop, loop)
    sample.start()                                                        
    loop.run()                                       
    sample.stop()                                 
